# -*- coding: iso-8859-1 -*-
# IMPORTANT! This encoding (charset) setting MUST be correct! If you live in a
# western country and you don't know that you use utf-8, you probably want to
# use iso-8859-1 (or some other iso charset). If you use utf-8 (a Unicode
# encoding) you MUST use: coding: utf-8
# That setting must match the encoding your editor uses when you modify the
# settings below. If it does not, special non-ASCII chars will be wrong.

"""
    MoinMoin - Configuration for a wiki farm

    If you run a single wiki only, you can keep the "wikis" list "as is"
    (it has a single rule mapping all requests to mywiki.py).

    Note that there are more config options than you'll find in
    the version of this file that is installed by default; see
    the module MoinMoin.multiconfig for a full list of names and their
    default values.

    Also, the URL http://moinmoin.wikiwikiweb.de/HelpOnConfiguration has
    a list of config options.
"""


# Wikis in your farm --------------------------------------------------

# If you run multiple wikis, you need this list of pairs (wikiname, url
# regular expression). moin processes that list and tries to match the
# regular expression against the URL of this request - until it matches.
# Then it loads the <wikiname>.py config for handling that request.

#print "Unsere kleine Farm"
# Important:
#  * the left part is the wikiname enclosed in double quotes
#  * the left part must be a valid python module name, so better use only
#    lower letters "a-z" and "_". Do not use blanks or "-" there!!!
#  * the right part is the url re, use r"..." for it
#  * the right part does NOT include "http://" nor "https://" at the beginning
#  * in the right part ".*" means "everything". Just "*" does not work like
#    for filenames on the shell / commandline, you must use ".*" as it is a RE.
#  * in the right part, "^" means "beginning" and "$" means "end"

wikis = [
    # Standalone server needs the port e.g. localhost:8000
    # Twisted server can now use the port, too.
    
    # wikiname,     url regular expression (no protocol)
    # ---------------------------------------------------------------
#    ("mywiki",  r".*"),   # this is ok for a single wiki

    # for multiple wikis, do something like this:
    #("moinmoin",    r"^moinmoin.wikiwikiweb.de/.*$"),
    #("moinmaster",  r"^moinmaster.wikiwikiweb.de/.*$"),
    ("master15",    r".*master15.*$"),
#    ("master13",    r".*master13.*$"),
#    ("master11",    r".*master11.*$"),
    ("orswiki",    r".*orswiki.*$"),
    ("referenceplant",    r".*referenceplant.*$"),
    ("devinfo",    r".*devinfo.*$"),
    ("schedule",    r".*schedule.*$"),
]


# Common configuration for all wikis ----------------------------------

# Everything that should be configured the same way should go here,
# anything else that should be different should go to the single wiki's
# config.
# In that single wiki's config, we will use the class FarmConfig we define
# below as the base config settings and only override what's different.
#
# In exactly the same way, we first include MoinMoin's Config Defaults here -
# this is to get everything to sane defaults, so we need to change only what
# we like to have different:

from MoinMoin.multiconfig import DefaultConfig
#this is needed for ORS smiley extension!
from MoinMoin import config
# Now we subclass this DefaultConfig. This means that we inherit every setting
# from the DefaultConfig, except those we explicitely define different.

#RS stop    
from MoinMoin.security import Permissions
from MoinMoin.Page import Page
class ORSSecurityPolicy(Permissions):
#RS continue

    def __init__(self, user):
        """ Calculate the permissons `user` has.
        """
        Permissions.__init__(self,user)
#        raise shit
        
    def _admin(self):
        """
        dirty hack for SystemAdmin macro
        """
        page_acl=self.getACL("FrontPage")
        allowed=page_acl.may(self.user.name, "admin")
##            self.user._request.write("<P><HR>ACL for %s at %s:\n<br>%s\ncheck do %s=%s\n<HR><P>" %(self.user.name,pagename,str(page_acl.acl),action,allowed))

        return allowed
        
    def save(self, editor, newtext, datestamp, **kw):
        """
        Check whether user may save a page.
        overriden to allow forced save
        @keyword force: ignore ACL and allow saving

        else uses default behaviour                  
        """
        force=kw.get('force',0)
        if force==1:
            return 1
        return Permissions.save(self, editor, newtext, datestamp, **kw)

    def do(self,action, pagename, **kw):
        """
        Check whether user may do "action" with this page.

        @keyword kw: not used currently.
        """
        
        page_acl=Page(self.request, pagename).getACL(self.request)
        allowed=page_acl.may(self.request,self.name, action)
##            self.user._request.write("<P><HR>ACL for %s at %s:\n<br>%s\ncheck do %s=%s\n<HR><P>" %(self.user.name,pagename,str(page_acl.acl),action,allowed))

        return allowed


class FarmConfig(DefaultConfig):

    sitename=u"FARM"
    page_front_page = u'FrontPage'

    # Critical setup  ---------------------------------------------------

    # Misconfiguration here will render your wiki unusable. Check that
    # all directories are accessible by the web server or moin server.

    # If you encounter problems, try to set data_dir and data_underlay_dir
    # to absolute paths.

    # Where your mutable wiki pages are. You want to make regular
    # backups of this directory.
    data_dir = './data/'
    shared_intermap = ['http://ors-x-wiki/master15cgi/moin.cgi/InterWikiMap?action=raw','http://ors-x-wiki/master15cgi/moin.cgi/InterWikiMapOrs?action=raw'] 

    # Where read-only system and help page are. You might want to share
    # this directory between several wikis. When you update MoinMoin,
    # you can safely replace the underlay directory with a new one. This
    # directory is part of MoinMoin distribution, you don't have to
    # backup it.
#    data_underlay_dir = './underlay/'
    # as it isnt modified, it can be share between all instances:
    data_underlay_dir = 'D:\\WEBServer\\Homepage\\moin\\underlay\\'

    # This must be '/wiki' for twisted and standalone. For CGI, it should
    # match your Apache Alias setting.
    url_prefix = '/wiki'
    url_prefix_dir = 'D:\\WEBServer\\Homepage\\moin15\\share\\moin\\htdocs\\'

#base target dir and url for exports. must be in a valid www realm, e.g. under /wiki
    export_prefix = '/wiki/exports'
    export_prefix_dir = 'D:\\WEBServer\\Homepage\\moin15\\share\\moin\\htdocs\\exports'

    # Security ----------------------------------------------------------

    # This is checked by some rather critical and potentially harmful actions,
    # like despam or PackageInstaller action:
    superuser = [u"RobertSeeger", ]

    # Security critical actions (disabled by default)
    # Uncomment to enable options you like.
    allowed_actions = ['DeletePage', 'AttachFile', 'RenamePage']
#RS extend SecurityPolicy class to allow access to all configured ACL actions by name

    
    # Enable acl (0 to disable)
    acl_enabled = 1    

    # IMPORTANT: grant yourself admin rights! replace YourName with
    # your user name. See HelpOnAccessControlLists for more help.
    # All acl_rights_xxx options must use unicode [Unicode]
    #acl_rights_before = u"YourName:read,write,delete,revert,admin"

    # Link spam protection for public wikis (uncomment to enable).
    # Needs a reliable internet connection.
    #from MoinMoin.util.antispam import SecurityPolicy

    edit_locking = 'warn 20' # None, 'warn <timeout mins>', 'lock <timeout mins>'

    # Mail --------------------------------------------------------------

    # Configure to enable subscribing to pages (disabled by default) or
    # sending forgotten passwords.

    # SMTP server, e.g. "mail.provider.com" (empty or None to disable mail)
    mail_smarthost='alpha'


    # The return address, e.g "My Wiki <noreply@mywiki.org>"
    mail_from = u""

    # "user pwd" if you need to use SMTP AUTH
    mail_login = ""


    # User interface ----------------------------------------------------

    # Add your wikis important pages at the end. It is not recommended to
    # remove the default links.  Leave room for user links - don't use
    # more than 6 short items.
    # You MUST use Unicode strings here, but you need not use localized
    # page names for system and help pages, those will be used automatically
    # according to the user selected language. [Unicode]
    navi_bar = [
        # Will use page_front_page, (default FrontPage)
        u'%(page_front_page)s',
        u'RecentChanges',
        u'FindPage',
        u'HelpContents',
##        u'[wiki:Schedule:FrontPage SCHEDULE V7]',
##        u'[wiki:Referenceplant:FrontPage REFPLANT]',
##        u'[wiki:Devinfo:FrontPage DEVINFO]',
##        u'[wiki:Orswiki:FrontPage ORSWIKI]',
    ]

    # The default theme anonymous or new users get
#    theme_default = 'modern'
    theme_default = 'monobook'


    # Language options --------------------------------------------------

    # See http://moinmoin.wikiwikiweb.de/ConfigMarket for configuration in 
    # YOUR language that other people contributed.

    # The main wiki language, set the direction of the wiki pages
    language_default = 'en'

    # You must use Unicode strings here [Unicode]
    page_category_regex = u'^Category[A-Z]'
    page_dict_regex = u'[a-z]Dict$'
    page_group_regex = u'[a-z]Group$'
    page_template_regex = u'[a-z]Template$'

    # Content options ---------------------------------------------------

    # Show users hostnames in RecentChanges
    show_hosts = 1
    show_version=1               
    # Show the interwiki name (and link it to page_front_page) in the Theme,
    # nice for farm setups or when your logo does not show the wiki's name.
    show_interwiki = 1
    logo_string = u''

    # Enumerate headlines?
    show_section_numbers = 0

    smileys = {
        "{ORS}":  (30, 30,0, "ors_logo.gif"),
        "{CUS}":  (60, 25,0, "customer.gif"),
        "{OLD}":  (60, 25,0, "old_g.gif"),
        "{PRE}":  (60, 25,0, "prep_g.gif"),
        "{RIGHT}":  (11,11,0, "arrowrt.gif"),
        "{DOWN}":  (11,11,0, "arrowdn.gif"),
        "{LINK}":  (11,11,0, "link_transp.gif"),
        "{DETAIL}":  (21,20,0, "detail_transp.gif"),
        "{CAUTION}":  (24,24,0, "caution.gif"),
        "{EXAMPLE}":  (32,32,0, "example.gif"),
        "{HINT}":  (24,24,0, "hint.gif"),
        "{RECOMMEND}":  (24,24,0, "recommendation.gif"),
        "{SYNTAX}":  (32,32,0, "syntax.gif"),
        "{AFS}": (36,18,0, "afs.gif")
    }
    orsicons={
        "0": ["12","12"],      
        "1": ["12","13"],      
        "10": ["14","13"],      
        "11": ["11","13"],      
        "12": ["13","14"],      
        "13": ["14","14"],      
        "14": ["12","13"],      
        "15": ["13","14"],      
        "16": ["11","12"],      
        "17": ["16","5"],      
        "18": ["8","5"],      
        "19": ["8","11"],      
        "2": ["12","14"],      
        "20": ["8","9"],      
        "21": ["16","9"],      
        "22": ["16","15"],      
        "23": ["16","15"],      
        "24": ["16","15"],      
        "25": ["16","14"],      
        "26": ["16","15"],      
        "27": ["16","15"],      
        "28": ["14","15"],      
        "29": ["15","15"],      
        "3": ["15","15"],      
        "30": ["16","15"],      
        "31": ["16","10"],      
        "32": ["16","15"],
        "33": ["16","15"],      
        "34": ["14","14"],      
        "35": ["16","15"],      
        "36": ["16","15"],      
        "37": ["16","15"],      
        "38": ["16","15"],      
        "39": ["14","14"],      
        "4": ["32","10"],      
        "40": ["14","14"],      
        "41": ["14","14"],      
        "42": ["14","14"],      
        "43": ["16","15"],      
        "44": ["16","14"],      
        "45": ["16","16"],
        "46": ["16","16"],
        "47": ["16","16"],
        "48": ["16","16"],
        "49": ["15","12"],      
        "5": ["32","10"],      
        "50": ["15","14"],      
        "51": ["16","15"],      
        "52": ["16","15"],      
        "53": ["16","15"],      
        "54": ["16","16"],      
        "55": ["12","13"],      
        "56": ["13","14"],      
        "57": ["14","14"],      
        "58": ["11","12"],      
        "59": ["10","10"],      
        "6": ["32","10"],      
        "60": ["10","10"],      
        "61": ["10","10"],      
        "62": ["10","10"],      
        "63": ["12","13"],      
        "64": ["12","14"],      
        "65": ["12","12"],      
        "66": ["16","15"],      
        "67": ["32","10"],      
        "68": ["32","10"],      
        "69": ["32","10"],      
        "7": ["32","10"],      
        "70": ["32","10"],      
        "71": ["21","10"],      
        "72": ["21","10"],      
        "73": ["13","15"],      
        "74": ["16","13"],      
        "75": ["13","14"],      
        "76": ["17","14"],      
        "77": ["11","8"],      
        "78": ["12","12"],      
        "79": ["12","12"],      
        "8": ["12","14"],      
        "80": ["12","12"],      
        "81": ["12","12"],      
        "82": ["14","14"],      
        "83": ["14","13"],      
        "9": ["13","14"],      
        }


    for icid,icdim in orsicons.items():
        i=int(icid)
        key="{ORS_%03i}" % i
        bmp="orsicon/%s.bmp" % icid
        smileys[key]=(int(icdim[0]),int(icdim[1]),0,bmp)
    config.smileys.update(smileys)



    # Enable graphical charts, requires gdchart.
    chart_options = {'width': 600, 'height': 300}


    #ACL
    acl_rights_valid= [u'read', u'write', u'delete', u'revert', u'admin',u'massdelete']
    acl_default_write=u'admin,read,write,delete,revert'
    acl_default_admin=u'massdelete'

    #default farm
    ##acl_rights_default='Known:admin,read,write,delete,revert All:read,write'
    #tougher: anonymous is read-only
    acl_rights_default=u'Known:%s' % (acl_default_write)
    acl_rights_before=u'WikiAdmin,RobertSeeger:%s,%s' % (acl_default_write,acl_default_admin)
    acl_rights_after=u'All:read'

    #predefined acl lines
    #replaces old "locked"
    acl_line_locked=u'All:read'

    #replaces old "locked_hard"
    acl_line_locked_hard=u'WikiAdminGroup:%s All:read' % (acl_default_write)

    #replaces old "locked_test"
    acl_line_locked_test=u'All:read'

#spider list modified to allow JensOehler (searchalot)
    ua_spiders = ('archiver|cfetch|crawler|curl|gigabot|google|holmes|htdig|httrack|httpunit|jeeves|larbin|leech|'
                  'linkbot|linkmap|linkwalk|mercator|mirror|msnbot|nutbot|puf|robot|scooter|'
                  'search(?!alot)|sherlock|sitecheck|spider|teleport|wget')

    hacks = { 'ie7': False}

##    page_header1="""
##        <SCRIPT type="text/vbscript" language="vbscript" src="/wiki13/applets/svghelper.vbs">
##        </SCRIPT>
##        <SCRIPT type="text/vbscript" language="vbscript" src="/wiki13/applets/dothelper.vbs">
##        </SCRIPT>
##    <script language="JavaScript" src="/wiki13/wedit/protoedit.js"></script>
##    <script language="JavaScript" src="/wiki13/wedit/wikiedit2.js"></script>
##    <script type="text/javascript">
##
##    function start()
##    {
##        wE = new WikiEdit();
##        wE.init('editor-textarea','WikiEdit','edname','/wiki13/wedit/images/');
##    //    alert('wysiwyngahaha');
##    }
##    </script>
##    """

    #this can be changed by individual wikis
    ##title2 = """

    def page_header2(self,request):

        title2_long = """
         <HR><P><B><I>This Wiki Site is for internal use only! Neither the handing over of this document (any page of this Wiki instance) in any form to other persons nor the copying of sections of this document to non-members of OR Soft Jaenicke GmbH or SBM GmbH is allowed without the prior written approval of Robert Seeger. The reader is warned that the information contained in this document will change at any time without prior notification.</I></B><P><HR>
         """
        title2_short = """
         <P><B><I>This Site is for internal use only!</I></B><P><HR>
         """
#        this=request.user
#        he=this.name
#        if he=='':
#            return title2_long;
#        else:
#            return title2_short
        return title2_short

##    page_footer1="""<br><br>
##    <a href="http://www.orsoft.net/"><img align="right" vspace="10" hspace="3"
##    src="%s/img/ors_logo.gif" width="31" height="31" border="0"
##    alt="OR Soft Jaenicke GmbH"></a>""" % (url_prefix,)

#DEVINFO add-ons
    external_databases={
         'XkeyForm': ['know','xkeystore','xkeystore',{}],
         'IdcKeyForm': ['know','xkeystore','xkeystore',{}],
         'DdicKeyForm': ['know','xkeystore','xkeystore',{}],
        }




    SecurityPolicy=ORSSecurityPolicy
