# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - "text/hhc" Formatter

    @copyright: 2000 - 2004 by Jrgen Hermann <jh@web.de>
    @license: GNU GPL, see COPYING for details.


    $Id: text_html.py,v 1.42 2002/05/10 11:39:01 jhermann Exp $
    ORS modifications:
        13.01.04 RS upgrade to 1.1.

        27.09.05 RS upgrade to 1.3.5, old code removed
"""
import os.path, re
from sets import Set # TODO: when we require Python 2.4+ use the builtin 'set' type
from MoinMoin.formatter.base import FormatterBase
from MoinMoin import wikiutil, i18n, config
from MoinMoin.formatter import text_html
from MoinMoin.Page import Page
from MoinMoin.action import AttachFile

line_anchors = False
prettyprint = True

target_encoding="cp1252"
encoding_errorhandler="ignore"

# These are the HTML elements that we treat as block elements.
_blocks = Set(['pagelink', ])
# These are the HTML elements which are typically only used with
# an opening tag without a separate closing tag.  We do not
# include 'script' or 'style' because sometimes they do have
# content, and also IE has a parsing bug with those two elements (only)
# when they don't have a closing tag even if valid XHTML.

_self_closing_tags = Set([])

# We only open those tags and let the browser auto-close them:
_auto_closing_tags = Set([])

# These are the elements which generally should cause an increase in the
# indention level in the html souce code.
_indenting_tags = Set(['UL','OL','LI','ol', 'ul', 'dl', 'li', 'dt', 'dd', 'tr', 'td'])

# These are the elements that discard any whitespace they contain as
# immediate child nodes.
_space_eating_tags = Set(['colgroup', 'dl', 'frameset', 'head', 'map' 'menu',
                          'ol', 'optgroup', 'select', 'table', 'tbody', 'tfoot',
                          'thead', 'tr', 'ul'])

# These are standard HTML attributes which are typically used without any
# value; e.g., as boolean flags indicated by their presence.
_html_attribute_boolflags = Set(['compact', 'disabled', 'ismap', 'nohref',
                                 'noresize', 'noshade', 'nowrap', 'readonly',
                                 'selected', 'wrap'])

# These are all the standard HTML attributes that are allowed on any element.
_common_attributes = Set(['accesskey', 'class', 'dir', 'disabled', 'id', 'lang',
                          'style', 'tabindex', 'title'])

_hhc_head="""
<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML//EN">\n
<HTML>\n
<HEAD>\n
<meta name="GENERATOR" content="Microsoft&reg; HTML Help Workshop 4.1">\n
<!-- Sitemap 1.0 -->\n
</HEAD><BODY>\n
"""

_hhc_foot="""
</BODY>
</HTML>
"""

class Formatter(text_html.Formatter):
    """
        Send HHC data.
    """

    hardspace = '&#160;'


    indentspace = ' '

    def __init__(self, request, **kw):
        apply(text_html.Formatter.__init__, (self, request), kw)
#RS additional property "mimetype"
        self.mimetype="text/hhc"
#RS end

        self._in_li = 0
        self._notext =1
        self._base_depth = 0
        self.store_text=0
        self.recordedtext=""





    def _open(self, tag, newline=False, attr=None, allowed_attrs=None, **kw):
        """ Open a tag with optional attributes (INTERNAL USE BY HTML FORMATTER ONLY!)
        
        @param tag: html tag, string
        @param newline: render tag so following data is on a separate line
        @param attr: dict with tag attributes
        @param allowed_attrs: list of allowed attributes for this element
        @param kw: arbitrary attributes and values
        @rtype: string ?
        @return: open tag with attributes as a string
        """
        # If it is self-closing, then don't expect a closing tag later on.
        is_self_closing = (tag in _self_closing_tags) and ' /' or ''

        if tag in _blocks:

            # Block elements
            self.store_text=1
            self.recordedtext=""
            result = []
            
            # Add language attributes, but let caller overide the default
            attributes = self._langAttr()
            if attr:
                attributes.update(attr)
            
            # Format
            attributes = self._formatAttributes(attributes, allowed_attrs=allowed_attrs, **kw)
            result.append('<%s%s%s>' % (tag, attributes, is_self_closing))
            if newline:
                result.append(self._newline())
            tagstr = ''.join(result)
        else:
            # Inline elements
            # Add to inlineStack
            if not is_self_closing:
                # Only push on stack if we expect a close-tag later
                self._inlineStack.append(tag)
            # Format
            tagstr = '<%s%s%s>' % (tag,
                                      self._formatAttributes(attr, allowed_attrs, **kw),
                                      is_self_closing)
        # XXX SENSE ???
        #if not self.close:
        #    self._tag_stack.append(tag)
        if tag in _indenting_tags:
             self._indent_level += 1
        return tagstr

    def _close(self, tag, newline=False):
        """ Close tag (INTERNAL USE BY HTML FORMATTER ONLY!)

        @param tag: html tag, string
        @param newline: render tag so following data is on a separate line
        @rtype: string
        @return: closing tag as a string
        """
        if tag in _self_closing_tags or tag in _auto_closing_tags:
            # This tag was already closed
            tagstr = ''
        elif tag in _blocks:
            # Block elements
            # Close all tags in inline stack
            # Work on a copy, because close(inline) manipulate the stack
            self.store_text=0
            self.recordedtext=""
            result = []
            stack = self._inlineStack[:]
            stack.reverse()
            for inline in stack:
                result.append(self._close(inline))
            # Format with newline
            if newline:
                result.append(self._newline())
            result.append('</%s>' % (tag))
            tagstr = ''.join(result)            
        else:
            # Inline elements 
            # Pull from stack, ignore order, that is not our problem.
            # The code that calls us should keep correct calling order.
            if tag in self._inlineStack:
                self._inlineStack.remove(tag)
            tagstr = '</%s>' % tag

        # XXX see other place marked with "SENSE"
        #if tag in _self_closing_tags:
        #    self._tag_stack.pop()
        if tag in _indenting_tags:
        # decrease indent level
            self._indent_level -= 1
        if newline:
            tagstr += self._newline()
        return tagstr

    # Public methods ###################################################

    def startDocument(self, pagename):
#        self.request.startRemoteDebugger()
        return _hhc_head
#        <UL>\n
##        <OBJECT type="text/site properties">\n
##            <param name="WindowName" value=" $global_main">\n
##        </OBJECT>\n 

    def endDocument(self):
#          </UL>
        return _hhc_foot
    def startContent(self, content_id='content', newline=True, **kw):
        """ Start page content div.

        A link anchor is provided at the beginning of the div, with
        an id of 'top' or 'top_xxxx' if the content_id argument is
        set to 'xxxx'.
        """
        return ""

    def endContent(self, newline=True):
        """ Close page content div.

        A link anchor is provided at the end of the div, with
        an id of 'bottom' or 'bottom_xxxx' if the content_id argument
        to the previus startContent() call was set to 'xxxx'.
        """
        """ Close page content div """
        return ""


    def lang(self, on, lang_name):
        """ Insert text with specific lang and direction.
        
            Enclose within span tag if lang_name is different from
            the current lang    
        """
        return ""
    # Links ##############################################################
    def pagelink(self, on, pagename='', page=None, **kw):
        """ Link to a page.

            formatter.text_python will use an optimized call with a page!=None
            parameter. DO NOT USE THIS YOURSELF OR IT WILL BREAK.

            See wikiutil.link_tag() for possible keyword parameters.
        """
        apply(FormatterBase.pagelink, (self, on, pagename, page), kw)
        if page is None:
            page = Page(self.request, pagename, formatter=self);
##        if self.request.export_mode:
##            if filter(lambda x,p=pagename:x==p,self.request.export_pages)==[]:
##                #just render as text?
##                kw['css_class'] = 'nonexistent'
##                return self.span(on, **kw)
        if not page.exists():
            #just do nothing?
            return ''
        targetdir='htm'
        targetdir='.'
##        try:
##            targetdir=request.cfg.dump_prefix
##        except:
##            pass
##        if page is None:
##            page = Page(self.request, pagename, formatter=self);
        tag = 'pagelink'
        if on:
##            tagstr=self._open(tag, attr={'class': 'pagelink'})
            tagstr=""
            # Block elements
            self.store_text=1
            self.recordedtext=""

        else:
            text=self.recordedtext
            if text==None or text=="":
                text=pagename
            self.store_text=0
            self.recordedtext=""
##            tagstr=self._close(tag)
            tagstr=""
            tagstr+= self._open("OBJECT", newline=1, type="text/sitemap")
            if self._in_li and self._notext and text!=None:
                s=wikiutil.escape(text).replace(' ', self.hardspace)
#                s=ss.encode(target_encoding,encoding_errorhandler)
#                self.request.startRemoteDebugger()

                tagstr+='\n            <param name="Name" value="%s">' % s
                tagstr+="""
                \n            <param name="Local" value="%s/%s.html">
                """ % (targetdir,pagename.replace("/","(2f)"))
                #tagstr+=tagstr2.encode(target_encoding,encoding_errorhandler)
            else:
                tagstr+="""
                \n            <param name="Local" value="%s/%s.html">
                """ % (targetdir,pagename.replace("/","(2f)"))
                #tagstr+=tagstr2.encode(target_encoding,encoding_errorhandler)
                
            tagstr +=  self._close("OBJECT", newline=1)
        return tagstr






    def url(self, on, url=None, css=None, do_escape=0, **kw):
        """
        Inserts an <a> element.

        Call once with on=1 to start the link, and again with on=0
        to end it (no other arguments are needed when on==0).

        do_escape: XXX doesn't work yet

        Keyword params:
            url - the URL to link to; will go through Wiki URL mapping.
            css - a space-separated list of CSS classes
            attrs -  just include this string verbatim inside
                     the <a> element; can be used for arbitrary attrs;
                     all escaping and quoting is the caller's responsibility.

        Note that the 'attrs' keyword argument is for backwards compatibility
        only.  It should not be used for new code -- instead just pass
        any attributes in as separate keyword arguments.

        1.5.3: removed ugly "attrs" keyword argument handling code
        """
        return ""
        
    def anchordef(self, id):
        """Inserts an invisible element used as a link target.

        Inserts an empty <span> element with an id attribute, used as an anchor
        for link references.  We use <span></span> rather than <span/>
        for browser portability.
        """
        return ''

    def anchorlink(self, on, name='', **kw):
        """Insert an <a> link pointing to an anchor on the same page.

        Call once with on=1 to start the link, and a second time with
        on=0 to end it.  No other arguments are needed on the second
        call.

        The name argument should be the same as the id provided to the
        anchordef() method, or some other elment.  It should NOT start
        with '#' as that will be added automatically.

        The id argument, if provided, is instead the id of this link
        itself and not of the target element the link references.
        """
        return ''

    # Text ##############################################################
    
    def _text(self, text):

        text = wikiutil.escape(text)
        if self._in_code:
            text = text.replace(' ', self.hardspace)
            self._notext=0
            text='\n            <param type="??" name="Name" value="%s">' % text
        if self.store_text==1:
            self.recordedtext+=text
            return ''
        else:
            return ''
        return text

    # Inline ###########################################################

    def _chardummy(self,on,**kw):
        return ''
        
    strong=emphasis=sub=sup=underline=highlight=small=big=code=strike=_chardummy    

    # Block elements ####################################################

    preformatted= code_line= _chardummy    
    def code_area(self, on, code_id, code_type='code', show=0, start=-1, step=-1):
        """Creates a formatted code region, with line numbering.

        This region is formatted as a <div> with a <pre> inside it.  The
        code_id argument is assigned to the 'id' of the div element, and
        must be unique within the document.  The show, start, and step are
        used for line numbering.

        Note this is not like most formatter methods, it can not take any
        extra keyword arguments.

        Call once with on=1 to start the region, and a second time
        with on=0 to end it.
        """
        return ""
    def code_token(self, on, tok_type):
        return ""

    # Paragraphs, Lines, Rules ###########################################

    def _indent_spaces(self):
        """Returns space(s) for indenting the html source so list nesting is easy to read.

        Note that this mostly works, but because of caching may not always be accurate."""
        if prettyprint:
            return self.indentspace * self._indent_level
        else:
            return ''

    def _newline(self):
        """Returns the whitespace for starting a new html source line, properly indented."""
        if prettyprint:
            return '\n' + self._indent_spaces()
        else:
            return ''
            
    def linebreak(self, preformatted=1):
        """Creates a line break in the HTML output.
        
        If preformatted is true a <br> element is inserted, otherwise
        the linebreak will only be visible in the HTML source.
        """
        return ''

    paragraph= div=span=_chardummy

    def rule(self, size=None, **kw):
        """Creates a horizontal rule with an <hr> element.

        If size is a number in the range [1..6], the CSS class of the rule
        is set to 'hr1' through 'hr6'.  The intent is that the larger the
        size number the thicker or bolder the rule will be.
        """
        return ''

    def icon(self, type):
        return ''

    def smiley(self, text):
        return ''

    # Lists ##############################################################


    def number_list(self, on, type=None, start=None, **kw):
        """Creates an HTML ordered list, <ol> element.

        The 'type' if specified can be any legal numbered
        list-style-type, such as 'decimal','lower-roman', etc.

        The 'start' argument if specified gives the numeric value of
        the first list item (default is 1).

        Call once with on=1 to start the list, and a second time
        with on=0 to end it.
        """
        return ['<UL>\n', '\n    </UL>'][not on]
        return ''

    def bullet_list(self, on, **kw):
        """Creates an HTML ordered list, <ul> element.

        The 'type' if specified can be any legal unnumbered
        list-style-type, such as 'disc','square', etc.

        Call once with on=1 to start the list, and a second time
        with on=0 to end it.
        """
        tag = 'UL'
        if on:
            tagstr = self._open(tag, newline=1, **kw)
        else:
            tagstr = self._close(tag, newline=1)
        return tagstr


    def listitem(self, on, **kw):
        """Adds a list item, <li> element, to a previously opened
        bullet or number list.

        Call once with on=1 to start the region, and a second time
        with on=0 to end it.
        """
        self._in_li = on != 0
        self._notext=1
        tag = 'LI'
        if on:
            tagstr =  self._open(tag, newline=1)
            
        else:
            tagstr = self._close(tag, newline=1)
        return tagstr
##        return ['\n   <LI> <OBJECT type="text/sitemap">', '\n    </OBJECT>'][not on]

    def definition_list(self, on, **kw):
        """Creates an HTML definition list, <dl> element.

        Call once with on=1 to start the list, and a second time
        with on=0 to end it.
        """
        return ''

    def definition_term(self, on, **kw):
        """Adds a new term to a definition list, HTML element <dt>.

        Call once with on=1 to start the term, and a second time
        with on=0 to end it.
        """
        return ''

    def definition_desc(self, on, **kw):
        """Gives the definition to a definition item, HTML element <dd>.

        Call once with on=1 to start the definition, and a second time
        with on=0 to end it.
        """
        return ''

    def heading(self, on, depth, **kw):
        return ''

    # Tables #############################################################



    def table(self, on, attrs=None, **kw):
        """ Create table

        @param on: start table
        @param attrs: table attributes
        @rtype: string
        @return start or end tag of a table
        """
        return ''

    def table_row(self, on, attrs=None, **kw):
        return ''

    def table_cell(self, on, attrs=None, **kw):

        return ''

    def text(self, text, **kw):
        txt = self._text(text)
        return txt

    def escapedText(self, text, **kw):
        txt = wikiutil.escape(text)
        if kw:
            return self._open('span', **kw) + txt + self._close('span')
        return txt

#RS taken from base
    def image(self, src=None, **kw):
        """An inline image.

        Extra keyword arguments are according to the HTML <img> tag attributes.
        In particular an 'alt' or 'title' argument should give a description
        of the image.
        """
        return ""




