# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - "text/html" Formatter

    Copyright (c) 2000, 2001, 2002 by Jrgen Hermann <jh@web.de>
    All rights reserved, see COPYING for details.

    $Id: text_html.py,v 1.42 2002/05/10 11:39:01 jhermann Exp $
    ORS modifications:
        13.01.04 RS upgrade to 1.1.

"""

# Imports
import cgi, string, sys, time
from MoinMoin.formatter.base import FormatterBase
#RS 1.1.
##from MoinMoin import wikiutil, config, user,webapi
from MoinMoin import wikiutil, config, user,webapi, i18n
#RS end

from MoinMoin.Page import Page
#RS 1.1.
##from MoinMoin.i18n import _
#RS end

#############################################################################
### HTML Formatter
#############################################################################

class Formatter(FormatterBase):
    """
        Send HHK data.
    """

    hardspace = '&#160;'

    _allowed_table_attrs = {
        'table': ['border', 'cellspacing', 'cellpadding', 'width', 'bgcolor', ],
        'row': ['width', 'align', 'valign', 'bgcolor', ],
        '': ['width', 'align', 'valign', 'colspan', 'rowspan', 'bgcolor', ],
    }
    _table_defaults = [
        ('tableborder', '"1"'),
        ('tablecellspacing', '"0"'),
        ('tablecellpadding', '"3"'),
    ]


    def __init__(self, request, **kw):
        apply(FormatterBase.__init__, (self, request), kw)
#RS additional property "mimetype"
        self.mimetype="text/hhk"
#RS end

        self._in_li = 0
        self._in_code = 0
        self._base_depth = 0
        self._show_section_numbers = None

        if not hasattr(request, '_fmt_hd_counters'):
            request._fmt_hd_counters = []

#RS 1.1.
    def _langAttr(self):
        result = ''
        lang = self.request.current_lang
        if lang != config.default_lang:
            result += ' lang="%s" dir="%s"' % (
                lang, i18n.getDirection(self.request, lang))

        return result
#RS end

    def startDocument(self, pagename):
        return """
        <!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML//EN">\n
        <HTML>\n
        <HEAD>\n
        <meta name="GENERATOR" content="Microsoft&reg; HTML Help Workshop 4.1">\n
        <!-- Sitemap 1.0 -->\n
        </HEAD><BODY>\n
        <UL>\n
        """
##        <OBJECT type="text/site properties">\n
##            <param name="WindowName" value=" $global_main">\n
##        </OBJECT>\n 

    def endDocument(self):
        return """
          </UL>
        </BODY>
        </HTML>
        """

    def sysmsg(self, text, **kw):
        return '<div class="message"><b>%s</b></div>' % cgi.escape(text)

    def pagelink(self, pagename, text=None, **kw):
        """ Link to a page.

            See wikiutil.link_tag() for possible keyword parameters.
        """
        targetdir='htm'
        try:
            targetdir=config.dump_prefix
        except:
            pass

        return ("""
        \n            <param name="Name" value="%s">
        \n            <param name="Local" value="%s/%s.html">
        """ % (pagename,targetdir,pagename.replace("/","_2f")))
#RS 1.1.
##        return Page(pagename).link_to(text)
        return Page(pagename).link_to(text, **kw)
#RS end


    def url(self, url, text=None, css=None, **kw):
        """
            Keyword params:
                title - title attribute
                target - target attribute
                ... some more (!!! TODO) 

        """
        return ""
        url = wikiutil.mapURL(url)
        pretty = kw.get('pretty_url', 0)
#RS 1.1.
        target = kw.get('target', None)
#RS end

        if not pretty and wikiutil.isPicture(url):
            return '<img src="%s" border="0">' % (url,)

        if text is None: text = url
        str = ''

#RS 1.1.
##        # add popup icon if user asked for it
##        if pretty and self.request.user.external_target:
##            str = ('%s<a target="_blank" href="%s"><img src="%s/img/moin-popup.gif"'
##                ' border="0" width="15" height="9" alt="%s"></a>') % (
##                str, cgi.escape(url, 1), config.url_prefix, _('[New window]'))
        # add popup icon if user asked for it or a target is set
        if pretty and (self.request.user.external_target or target is not None):
            str = ('%s<a target="_blank" href="%s"><img src="%s/img/moin-popup.gif"'
                ' border="0" width="15" height="9" alt="%s" title="%s"></a>') % (
                str, cgi.escape(url, 1), config.url_prefix, self._('[New window]'), self._('[New window]'))
#RS end

        # create link
        str = str + '<a'
        if css: str = '%s class="%s"' % (str, css)
        title = kw.get('title', None)
        if title: str = '%s title="%s"' % (str, title)
#RS 1.1.
        if target: str = '%s target="%s"' % (str, target)
#RS end

        str = '%s href="%s">%s</a>' % (str, cgi.escape(url, 1), text)

        return str

    def text(self, text):
        if self._in_code:
            s=string.replace(cgi.escape(text), ' ', self.hardspace)
            return ('\n            <param name="Name" value="%s">' % s)

        return cgi.escape(text)

    def rule(self, size=0):
        return ''
        if size:
            return '<hr size="%d">\n' % (size,)
        else:
            return '<hr>\n'

    def strong(self, on):
        return ''
        return ['<b>', '</b>'][not on]

    def emphasis(self, on):
        return ''
        return ['<em>', '</em>'][not on]

    def highlight(self, on):
        return ''
        return ['<strong class="highlight">', '</strong>'][not on]

    def number_list(self, on, type=None, start=None):
        return ''
        if not on: return '</ol>'

        result = '<ol'
        if type: result = result + ' type="%s"' % (type,)
#RS 1.1.
##        if start: result = result + ' start="%d"' % (start,)
        if start is not None: result = result + ' start="%d"' % (start,)
#RS end
        return result + '>'

    def bullet_list(self, on):
        return ['\n', '\n    </OBJECT>'][not on]

#RS 1.1.
##    def listitem(self, on):
    def listitem(self, on, **kw):
#RS end
        self._in_li = on != 0
        return ['', ''][not on]

    def sup(self, on):
        return ''
        return ['<sup>', '</sup>'][not on]

#RS 1.1.
    def sub(self, on):
        return ''
        return ['<sub>', '</sub>'][not on]
#RS end

    def code(self, on):
        self._in_code = on
        
        return ['\n  <LI><OBJECT type="text/sitemap">', '\n'][not on]

    def preformatted(self, on):
        return ''
        FormatterBase.preformatted(self, on)
        return ['<pre class="code">', '</pre>'][not on]

    def paragraph(self, on):
        return ''
        FormatterBase.paragraph(self, on)
        if self._in_li:
            self._in_li = self._in_li + 1
#RS 1.1.
##            return ['', '<p>'][on and self._in_li > 2]
##        else:
##            return ['<p>', ''][not on]
            return ['', '<p%s>' % self._langAttr()][on and self._in_li > 2]
        else:
            return ['<p%s>' % self._langAttr(), ''][not on]
#RS end

    def linebreak(self, preformatted=1):
        return ''
        return ['\n', '<br>'][not preformatted]

    def heading(self, depth, title, **kw):
        return ''
#RS 1.1.
##        # remember depth of first heading, and adapt current depth accordingly
##        if not self._base_depth:
##            self._base_depth = depth
##        depth = max(depth - (self._base_depth - 1), 1)
        # remember depth of first heading, and adapt counting depth accordingly
        if not self._base_depth:
            self._base_depth = depth
###
###RS adapt base depth if included
##            self._base_depth=long(self.request.getPragma('_base_depth',str(self._base_depth)))
##        oridepth=depth
##
###RS isn't this nonsense???
###        depth = max(depth - (self._base_depth - 1), 1)
##
##        depth = max(depth + (self._base_depth - 1), self._base_depth)
##
###            

        count_depth = max(depth - (self._base_depth - 1), 1)
#RS stop

        # check numbering, possibly changing the default
        if self._show_section_numbers is None:
            self._show_section_numbers = config.show_section_numbers
            numbering = string.lower(self.request.getPragma('section-numbers', ''))
            if numbering in ['0', 'off']:
                self._show_section_numbers = 0
            elif numbering in ['1', 'on']:
                self._show_section_numbers = 1
#RS continue
            elif numbering in ['2', '3', '4', '5', '6']:
                # explicit base level for section number display
                self._show_section_numbers = int(numbering)
#RS stop

        # create section number
        number = ''
        if self._show_section_numbers:
            # count headings on all levels
#RS continue
##            self.request._fmt_hd_counters = self.request._fmt_hd_counters[:depth]
##            while len(self.request._fmt_hd_counters) < depth:
            self.request._fmt_hd_counters = self.request._fmt_hd_counters[:count_depth]
            while len(self.request._fmt_hd_counters) < count_depth:
#RS stop
                self.request._fmt_hd_counters.append(0)
            self.request._fmt_hd_counters[-1] = self.request._fmt_hd_counters[-1] + 1
#RS continue
##            number = string.join(map(str, self.request._fmt_hd_counters), ".") + " "
##
##        return '<H%d>%s%s%s</H%d>\n' % (
##            depth, kw.get('icons', ''), number, title, depth)
            number = '.'.join(map(str, self.request._fmt_hd_counters[self._show_section_numbers-1:]))
            # CNC:2003-05-30
            if number: number += ". "

        if kw.has_key('on'):
            if kw['on']:
                return '<H%d>' % depth
            else:
                return '</H%d>' % depth
        else:
            return '<H%d%s>%s%s%s</H%d>\n' % (
                depth, self._langAttr(), kw.get('icons', ''), number, title, depth)
#RS end


    def _checkTableAttr(self, attrs, prefix):
        if not attrs: return ''

        result = ''
        for key, val in attrs.items():
            if prefix and key[:len(prefix)] != prefix: continue
            key = key[len(prefix):]
            if key not in self._allowed_table_attrs[prefix]: continue
            result = '%s %s=%s' % (result, key, val)

        return result

    def table(self, on, attrs={}):
        if on:
            attrs = attrs and attrs.copy() or {}
            for key, val in self._table_defaults:
                if not attrs.has_key(key): attrs[key] = val
            on = '<table class="wiki"%s>' % self._checkTableAttr(attrs, 'table')
        return ''
        return [on, '</table>'][not on]

    def table_row(self, on, attrs={}):
        if on:
            on = '<tr class="wiki"%s>' % self._checkTableAttr(attrs, 'row')
        return ''
        return [on, '</tr>'][not on]

    def table_cell(self, on, attrs={}):
        if on:
#RS 1.1.
##            on = '<td class="wiki"%s>' % self._checkTableAttr(attrs, '')
            on = '<td class="wiki"%s%s>' % (
                self._langAttr(),
                self._checkTableAttr(attrs, ''))
#RS end
        return ''
        return [on, '</td>'][not on]

    def anchordef(self, name):
        return ''
        return '<a name="%s"></a>' % name

    def anchorlink(self, name, text):
        return ''
        return '<a href="#%s">%s</a>' % (name, cgi.escape(text))

    def underline(self, on):
        return ''
        return ['<u>', '</u>'][not on]

    def definition_list(self, on):
        return ''
        return ['<dl>', '</dl>'][not on]

    def definition_term(self, on, compact=0):
        return ''
        extra = ''
        if compact:
            extra = ' compact'
#RS 1.1.
##        return ['<dt%s><b>' % (extra,), '</b></dt>'][not on]
        return ['<dt%s%s><b>' % (extra, self._langAttr()), '</b></dt>'][not on]
#RS end

    def definition_desc(self, on):
        return ''
#RS 1.1.
##        return ['<dd>', '</dd>'][not on]
        return ['<dd%s>' % self._langAttr(), '</dd>'][not on]
#RS end

