# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - Data associated with a single Request

    @copyright: 2001-2003 by Jrgen Hermann <jh@web.de>
    @copyright: 2003-2004 by Thomas Waldmann
    @license: GNU GPL, see COPYING for details.

    ORS specific request classes    

    ORS modifications::
        31.01.05 RS encapsulate IIS-patches and export mode in special classes
        03.02.05 RS all critical ORS modifications now check for request.ors_mod_active

    general enhancements::
    * page variables:
      * if pragma key is var_xyz, append key 'xyz' to self.pagevars
      * variables are evaluated for $var_xyz in Interwiki-Resolution in wiki.py
    * keyword store for export details etc. 
"""
import sys,os, mimetypes, time, urllib,shutil
from MoinMoin import config, search, wikiutil
from MoinMoin.Page import Page
from MoinMoin.request import RequestCGI,RequestCLI,RequestStandAlone
from MoinMoin.util import MoinMoinNoFooter, filesys, web

from MoinMoin.action import AttachFile



class RequestORSCGI(RequestCGI):
    """
    specialized on CGI requests
    if self.ors_mod_active is 0, behaves just like RequestCGI but allows extended call syntax
    """

    ors_mod_active=1
    def __init__(self, properties={}):
        RequestCGI.__init__(self, properties)
        self.keywords={}
        self.export_mode=0
        self.export_pages=[]
        self.pagevars=[]

#    def write(self, *data):
#        """ Write to output stream.
#        """
#        sys.stdout.write(data)
    def debugencode(self,text):
        """create a raw encode version of text that does not hide its real value due to unicode mangling
        """
        return (str(type(text))+"#".join(map(lambda x:str(hex(ord(x))),text)))

    def initTheme(self):
        """ Set theme - forced theme, user theme or wiki default """
        if self.cfg.theme_force:
            theme_name = self.cfg.theme_default
        else:
            theme_name = self.user.theme_name
        self.loadTheme(theme_name)

    def getScriptname(self,patch=0):
        """ Return the scriptname part of the URL ('/path/to/my.cgi'). """
        if self.ors_mod_active!=1:
            return RequestCGI.getScriptname(self)
            
        if self.script_name == '/':
            return ''
        iis_server=self.server_name
        name=self.script_name
#        if patch and iis_server in ['sapits','ors-x-wiki','heylpc','ors-x-know']:
#        if patch and iis_server in ['sapits','ors-x-wiki','heylpc']:
        if patch and iis_server in ['sapits','heylpc']:
            oldname=name
            name=oldname.replace('cgi/moin11.cgi','')
            name=oldname.replace('cgi11/moin11.cgi','')
            name=oldname.replace('cgi/moin.cgi','')
        return name

    def setPragma(self, key, value):
        """ Set a pragma value (#pragma processing instruction)

            Keys are not case-sensitive.
        """
        if self.ors_mod_active!=1:
            RequestCGI.setPragma(self, key, value)
        self.pragma[key.lower()] = value
        if key.startswith("var_"):
            varname=key.replace("var_","")
            self.pagevars.append(varname)

    def getAttachUrl(self,pagename, filename,addts=0, escaped=0):

        attach_dir = AttachFile.getAttachDir(self, pagename, create=0)
        pname = wikiutil.quoteWikinameFS(pagename)
        attach_dir = os.path.join(".", pname, "attachments")

        url = "%s/%s" % (attach_dir,
#        url = "%s/%s/attachments/%s" % (
#            request.getScriptname(), wikiutil.quoteWikinameFS(pagename),
            urllib.quote(filename.encode(config.charset)))
        return url

    def exportAttachments(self,pagename):
        """
        export all attachments for page
        """
        return
    
        files=AttachFile._get_files(self, pagename)
        attach_dir = AttachFile.getAttachDir(self, pagename, create=1)
        pname = wikiutil.quoteWikinameFS(pagename)
        attach_dir = os.path.join(self.export_dir, pname, "attachments")
        if len(files)==0:
            return
        if not os.path.isdir(attach_dir): 
            filesys.makeDirs(attach_dir)
        for filename in files:
            if filename.endswith('.chm'):
                continue
            fpath = AttachFile.getFilename(self, pagename, filename)
#            self.write('Writing attachment "%s" to "%s"...' % (fpath,self.export_dir))
            tpath=os.path.join(attach_dir,filename)
            shutil.copyfile( fpath, tpath)

    def startRemoteDebugger(self, timeout=300):           
        import rpdb2;
        rpdb2.start_embedded_debugger(pwd= "pw", fAllowUnencrypted= True, fRemote= True, timeout= 300)

class RequestORSCGIS(RequestORSCGI):
    """
    specialized on CGI requests for embedding in SCHEDULE++ (force theme)
    if self.ors_mod_active is 0, behaves just like RequestCGI but allows extended call syntax
    """

    ors_mod_active=1
    def __init__(self, properties={}):
        RequestORSCGI.__init__(self, properties)
 
    def initTheme(self):
        """ Set theme - forced theme, user theme or wiki default """
        self.loadTheme("schedule")


class RequestORSStandAlone(RequestStandAlone):
    """
    specialized on StandAlone Server (MoinMoin.server.standalone) requests
    if self.ors_mod_active is 0, behaves just like RequestCGI but allows extended call syntax
    """

    ors_mod_active=1
    def __init__(self, sa, properties={}):
        """
        @param sa: stand alone server object
        @param properties: ...
        """
        RequestStandAlone.__init__(self, sa, properties)
        self.keywords={}
        self.export_mode=0
        self.pagevars=[]

    def getScriptname(self,patch=0):
        """ Return the scriptname part of the URL ('/path/to/my.cgi'). """
        return RequestStandAlone.getScriptname(self)
            
##        if self.script_name == '/':
##            return ''
##        iis_server=self.server_name
##        name=self.script_name
###        if patch and iis_server in ['sapits','ors-x-wiki','heylpc','ors-x-know']:
##        if patch and iis_server in ['sapits','ors-x-wiki','heylpc']:
##            oldname=name
##            name=oldname.replace('cgi/moin11.cgi','')
##            name=oldname.replace('cgi11/moin11.cgi','')
##            name=oldname.replace('cgi/moin.cgi','')
##        return name

    def setPragma(self, key, value):
        """ Set a pragma value (#pragma processing instruction)

            Keys are not case-sensitive.
        """
        if self.ors_mod_active!=1:
            RequestStandAlone.setPragma(self, key, value)
        self.pragma[key.lower()] = value
        if key.startswith("var_"):
            varname=key.replace("var_","")
            self.pagevars.append(varname)
        


class RequestExportCLI(RequestCLI):
    """ specialized on export requests """

    ors_mod_active=1
    def __init__(self, url='CLI', pagename='', properties={}):
        #self.url=properties.get("url","unknownurl")
        RequestCLI.__init__(self, url,pagename,properties)
        self.export_mode=1
        self.export_dir="."
        self.export_pages=[]
        self.keywords={}
        self.pagevars=[]
        self.loadTheme('modern')
#        sys.stderr.write("request.theme=%s" % str(self.theme))

        d = {'title_text': None, 'title_link': None, 'page': None,}
        self.themedict = d
        self.setPragma('section-numbers', '0')
        

    def getScriptname(self,patch=0):
        """ Return the scriptname part of the URL ('/path/to/my.cgi'). """
        return '.'
        if self.script_name == '/':
            return ''
        iis_server=self.server_name
        name=self.script_name
#        if patch and iis_server in ['sapits','ors-x-wiki','heylpc','ors-x-know']:
        if patch and iis_server in ['sapits','heylpc']:
            oldname=name
            name=oldname.replace('cgi/moin11.cgi','')
            name=oldname.replace('cgi11/moin11.cgi','')
            name=oldname.replace('cgi/moin.cgi','')
        return name
        
    def setPragma(self, key, value):
        """ Set a pragma value (#pragma processing instruction)

            Keys are not case-sensitive.
        """
        if self.ors_mod_active!=1:
            RequestCLI.setPragma(self, key, value)
        self.pragma[key.lower()] = value
        if key.startswith("var_"):
            varname=key.replace("var_","")
            self.pagevars.append(varname)

    def getPragma(self, key, defval=None):
        """ Query a pragma value (#pragma processing instruction)

            Keys are not case-sensitive.
        """
        if key=='section-numbers':
            return '0'
        return self.pragma.get(key.lower(), defval)

    def http_redirect(self, url):
        """ Redirect to a fully qualified, or server-rooted URL """
        raise Exception("Redirect to #%s# not handled by exporter!" % url)

    def setup_args(self, form=None):
        args = {
            'action':['print'],
#            'media':['exporter'],
            'media':['projection'],
            }
        return args

    def getAttachUrl(self,pagename, filename,addts=0, escaped=0):

        attach_dir = AttachFile.getAttachDir(self, pagename, create=0)
        pname = wikiutil.quoteWikinameFS(pagename)
        attach_dir = os.path.join(".", pname, "attachments")

        url = "%s/%s" % (attach_dir,
#        url = "%s/%s/attachments/%s" % (
#            request.getScriptname(), wikiutil.quoteWikinameFS(pagename),
            urllib.quote(filename.encode(config.charset)))
        return url


    def exportAttachments(self,pagename):
        """
        export all attachments for page
        """
        files=AttachFile._get_files(self, pagename)
        attach_dir = AttachFile.getAttachDir(self, pagename, create=1)
        pname = wikiutil.quoteWikinameFS(pagename)
        attach_dir = os.path.join(self.export_dir, pname, "attachments")
        if len(files)==0:
            return
        if not os.path.isdir(attach_dir): 
            filesys.makeDirs(attach_dir)
        for filename in files:
            if filename.endswith('.chm'):
                continue
            fpath = AttachFile.getFilename(self, pagename, filename)
#            self.write('Writing attachment "%s" to "%s"...' % (fpath,self.export_dir))
            tpath=os.path.join(attach_dir,filename)
            shutil.copyfile( fpath, tpath) 

    def startRemoteDebugger(self, timeout=300):           
        import rpdb2;
        rpdb2.start_embedded_debugger(pwd= "pw", fAllowUnencrypted= True, fRemote= True, timeout= 300)

