# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - Export wiki pages

    based on::
        MoinMoin - Dump a MoinMoin wiki to static pages

        You must run this script as owner of the wiki files, usually this is the
        web server user.

        @copyright: 2002-2004 by Jrgen Hermann <jh@web.de>,
                    2005-2006 by Thomas Waldmann
        @license: GNU GPL, see COPYING for details.

    @copyright: 2004-2006 by Robert Seeger <rs@orsoft.de>,

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    ORS modifications:

    12.01.04 RS upgrade to 1.1., mark all previous ORS changes
    20.01.04 RS repair indent errors
    29.06.04 RS handle KeyboardInterrupt
    07.09.05 RS upgrade to 1.3.5
    24.09.05 RS use StringIO to buffer output, enable postprocessing
    04.11.05 RS new format "text/txw"
    12.06.06 RS refactor for 1.5.3 new script environment
            
"""
__version__ = "20060612"

#############################################################################
### Helpers
#############################################################################

##HTML_SUFFIX = ".html"  # perhaps put this in config.py as html_suffix?

logo_html = '<img src="moinmoin.png">'

url_prefix = "."

page_template = u'''<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<html>
<head>
<meta http-equiv="content-type" content="text/html; charset=%(charset)s">
<title>%(pagename)s</title>
<link rel="stylesheet" type="text/css" media="all" charset="utf-8" href="%(theme)s/css/common.css">
<link rel="stylesheet" type="text/css" media="screen" charset="utf-8" href="%(theme)s/css/screen.css">
<link rel="stylesheet" type="text/css" media="print" charset="utf-8" href="%(theme)s/css/print.css">
</head>
<body>
<table>
<tr>
<td>
%(logo_html)s
</td>
<td>
%(navibar_html)s
</td>
</tr>
</table>
<hr>
%(pagehtml)s
<hr>
%(timestamp)s
</body>
</html>
'''

SUFFIX={'html':".html",'plain':".txt",'word':".docdmp",'hhk':".hhk",'hhc':".hhc",'tal':".html",'xxml':".xml",'pdf':".pdfdmp",'txw':".txwdmp",'python':'.htm'}
DOCTYPE={'plain':"txt",'word':"doc",'pdf':"pdf",'python':"python"}
#############################################################################
### Main program
#############################################################################
import sys,re,string
import os, time, StringIO, codecs, shutil
from MoinMoin import config, search, wikiutil, Page
from MoinMoin.script import _util
from MoinMoin.script._util import MoinScript
from MoinMoin.request_ors import RequestExportCLI

##class MoinExporter(_util.Script):
class PluginScript(MoinScript):
    """ Dump script class """
##    def __init__(self):
##        _util.Script.__init__(self, __name__, "[options] <target-directory>")
    def __init__(self, argv=None, def_values=None):
        MoinScript.__init__(self, argv, def_values)
        self.pageoptions={}
        self.exp_format="html"
#        self.request=None
##        # --config=DIR            
##        self.parser.add_option(
##            "--config", metavar="DIR", dest="configdir",
##            help="Path to wikiconfig.py (or its directory)"
##        )
##
##        # --wiki=URL             
##        self.parser.add_option(
##            "--wiki", metavar="WIKIURL", dest="wiki_url",
##            help="URL of wiki to dump (e.g. moinmaster.wikiwikiweb.de)"
##        )

##        # --page=NAME             
##        self.parser.add_option(
##            "--page", metavar="NAME", dest="page",
##            help="Dump a single page (with possibly broken links)"
##        )

        self.parser.add_option(
            "-t", "--target-dir", dest="target_dir",
            help="Write export to DIRECTORY"
        )
        
#RS additional options
        # --pattern=REGEX             
        self.parser.add_option(
            "--pattern", metavar="PATTERN", dest="pattern",
            help="Export all pages with this pattern"
        )
        # --format=FORMAT             
        self.parser.add_option(
            "--format", metavar="FORMAT", dest="exportformat",
            help="Use export format"
        )
#RS end
#SYNC0
#RS refactored: separated code parts from parsing parts
    def setOutputDir(self,outdir):
        """
           set and create output dir
        """
        self.outputdir = os.path.abspath(outdir)
        if not os.path.isdir(self.outputdir):
            try:
                os.mkdir(self.outputdir)
                _util.log("Created output directory '%s'!" % self.outputdir)
            except OSError:
                _util.fatal("Cannot create output directory '%s'!" % self.outputdir)
        self.request.export_dir=self.outputdir

    def setFormatter(self,exp_format="html"):
        """
           load formatter
        """
        from MoinMoin import wikiutil
        # try to load the formatter
        mimetype = u"text/%s" % exp_format
##        print "format set to %s" % mimetype
#RS extension handling
        self.ext=SUFFIX.get(string.split(mimetype,"/")[1],'.html')
        self.doctype=DOCTYPE.get(string.split(mimetype,"/")[1],exp_format)
#        qfn_file = lambda pagename, qfn=wikiutil.quoteWikiname: qfn(pagename) + self.ext

        #modify behavior of wikiutil.quoteWikinameURL to add the extension 
        wikiutil.quoteWikinameURL = lambda pagename, qfn=wikiutil.quoteWikinameFS: (qfn(pagename)) + self.ext
#RS stop

        Formatter=None
        Formatter = wikiutil.importPlugin(self.request.cfg, "formatter",
            mimetype.translate({ord(u'/'): u'_', ord(u'.'): u'_'}), "Formatter")
#        Formatter = pysupport.importName("MoinMoin.formatter." +
#            string.translate(mimetype, string.maketrans('/.', '__')), "Formatter")
        if Formatter is None:
            # default to plain text formatter
#            sys.stderr.write("\nfallback to text/html!")
            del Formatter
            mimetype = "text/html"
            from MoinMoin.formatter.text_html import Formatter
    
#        self.pageoptions["formatter"]=Formatter(self.request,targetdir=self.outputdir)
#        self.pageoptions["formatter"]=Formatter(self.request)
#        self.pageoptions["export_mode"]=1
#        self.formatter=self.pageoptions["formatter"]
#        sys.stderr.write("\nformatter:%s" % str(Formatter))
        self.formatter=Formatter(self.request,targetdir=self.outputdir)
        self.pageoptions["formatter"]=self.formatter
    
        self.exp_format=exp_format
#        sys.stderr.write("\nself.formatter:%s" % str(self.formatter))
        self.target_charset=config.charset
        if exp_format in ["hhc","hhk"]:
            self.target_charset="cp1252"
    
    def write(self, *data):
        """ Write to output stream.
        """
        for piece in data:
            self.out.write(piece)



    def dumpPage(self,pagename,targetfile):
        """
           export the single page
        """
        from MoinMoin import Page
#RS attachment handling
        from MoinMoin.action import AttachFile
#RS end
        _util.log('Writing page "%s" to "%s"...' % (pagename.encode(config.charset),targetfile))
#RS output
        #self.request.write('Writing page "%s" to "%s"...' % (pagename,targetfile))
#        return
#RS end
        self.request.pragma={}

        errcnt=0
        request=self.request
        errlog=self.errlog
        outputdir=self.outputdir
        filepath = os.path.join(self.outputdir, targetfile)
#        self.out = codecs.open(filepath, 'w', config.charset)
        self.out=StringIO.StringIO()
        self.result=""
#        self.normwrite=self.request.write
        try:
#RS options
#            page = Page.Page(self.request,pagename,**self.pageoptions)
            page = Page.Page(request, pagename,**self.pageoptions)
#RS end
#    
            try:
                request.reset()
                request.sent_headers = 1 #avoid http headers in output
#                out = StringIO.StringIO()
                request.redirect(self.out)
                page.send_page(request, count_hit=0, content_only=0)
#                pagehtml = out.getvalue()
                request.exportAttachments(pagename)
                request.redirect()
            except KeyboardInterrupt:
                print >>sys.stderr, "*** keyboard interrupt,terminating while writing page %s!" % pagename
                print >>self.errlog, "~" * 78
                request.redirect()
                self.errlog.close()
                sys.exit(1)
            except SystemError,e:
                print >>sys.stderr, "*** Caught SystemError while writing page!"
                print >>errlog, "~" * 78
                print >>errlog, targetfile # page filename
                request.redirect()
                self.errlog.close()
                return page.send_page(request,
                    msg='<strong>%s</strong>' %
                        'SystemError %s' %(e))
#                raise SystemError,e
            except:
                errcnt = errcnt + 1
                print >>sys.stderr, "*** Caught exception while writing page!"
                print >>errlog, "~" * 78
                print >>errlog, targetfile # page filename
                import traceback
                traceback.print_exc(None, errlog)
                request.redirect()

#
#            sys.stdout = self.out
#            self.request.write=self.write
#            try:
#                page.send_page(self.request)
#RS handle KeyboardInterrupt
##            except KeyboardInterrupt:
##                print >>sys.stderr, "*** keyboard interrupt,terminating while writing page %s!" % pagename
##                print >>self.errlog, "~" * 78
##                self.out.close()
##                sys.stdout = sys.__stdout__
##                self.request.write=self.normwrite
##                self.errlog.close()
##                sys.exit(1)
###RS end            
##            except:
##                errcnt = errcnt + 1
##                print >>sys.stderr, "*** Caught exception while writing page %s!" % pagename
##                print >>self.errlog, "~" * 78
##                import traceback
##                traceback.print_exc(None, self.errlog)
#RS word cleanup
#                Formatter=self.pageoptions.get("formatter",None)
                Formatter=self.formatter
                if Formatter!=None:
                    if vars(Formatter).has_key('word_host'):
                        if Formatter.word_host!=None:
                            Formatter.word_host.Quit()
        finally:
#            timestamp = time.strftime("%Y-%m-%d %H:%M")
#            filepath = os.path.join(outputdir, file)
#            fileout = codecs.open(filepath, 'w', config.charset)
#            fileout.write(page_template % {
##                'charset': config.charset,
##                'pagename': pagename,
##                'pagehtml': pagehtml,
##                'logo_html': logo_html,
##                'navibar_html': navibar_html,
##                'timestamp': timestamp,
##                'theme': request.cfg.theme_default,
#            })
#            fileout.close()

            self.result=self.out.getvalue()
            self.out.close()
##            sys.stdout = sys.__stdout__
##            self.request.write=self.normwrite
#RS attachment
#            AttachFile.dump_filelist(self.request, pagename,os.path.dirname(self.outputdir))
#RS end

#RS end
#        if errcnt:
#            print >>sys.stderr, "*** %d error(s) occurred, see '%s'!" % (errcnt, errfile)
        self.outf = codecs.open(filepath, 'w', self.target_charset)
        self.outf.write(self.result)
        self.outf.close()
        return self.result

    def postprocessPage(self,pagename,file,output):
        """
        process the dumpPage result
        """
        outputdir=self.outputdir

        targetfile=file+"_src.py"
        filepath = os.path.join(self.outputdir, targetfile)

        if self.exp_format=="pdf":
            pass
        if self.exp_format=="python":
            pass
            src = self.formatter.assemble_code(output)
            outf = codecs.open(filepath, 'w', config.charset)
            outf.write(src)
            outf.close()
            
        
        return



    def mainloop(self):
        """ moin-exporter main code.
        """

##        if len(sys.argv) == 1:
##            self.parser.print_help()
##            sys.exit(1)
##
##        if len(self.args) != 1:
##            self.parser.error("incorrect number of arguments")

        #
#RS additional options
        if self.options.pattern:
            page_pattern = self.options.pattern
        else:
            page_pattern=".*"
       
        if self.options.exportformat:
            exp_format = self.options.exportformat
        else:
            exp_format = "html"

        

##        if configdir:
##            if os.path.isfile(configdir):
##                configdir = os.path.dirname(configdir)
##            if not os.path.isdir(configdir):
##                _util.fatal("Bad path given to --config parameter")
##            configdir = os.path.abspath(configdir)
##            sys.path[0:0] = [configdir]
##            os.chdir(configdir)
        # Insert config dir or the current directory to the start of the path.
        config_dir = self.options.config_dir
        if config_dir and os.path.isfile(config_dir):
            config_dir = os.path.dirname(config_dir)
        if config_dir and not os.path.isdir(config_dir):
            _util.fatal("bad path given to --config-dir option")
        sys.path.insert(0, os.path.abspath(config_dir or os.curdir))

        # fix some values so we get relative paths in output html
        # XXX maybe this should be configurable
#        config.url_prefix = "../wiki"
#        config.css_url    = "../wiki/css/moinmoin.css"
#RS output
##        print "dumping wiki site:",config.sitename," using pattern:",page_pattern
#RS end

        #
        # Dump the wiki
        
#        sys.stderr.write("\n\nself.options.wiki_url:%s" % self.options.wiki_url)

        self.init_request()
        request = self.request

        # fix url_prefix so we get relative paths in output html
        self.request.cfg.url_prefix = url_prefix
        self.cfg=request.cfg
        self.request.loadTheme("monobook")
        # Prepare output directory
        #
#RS refactored code
        # Prepare output directory
##        self.setOutputDir(self.args[0])
        self.setOutputDir(self.options.target_dir)
#RS end
        #
        from MoinMoin import wikiutil
        
#RS end


#    
#SYNC3

#RS refactored code
        self.setFormatter(exp_format)
        if exp_format in ["word","pdf","txw"]:    
            self.request.cfg.default_markup="wiki_word"
#RS end


##        print "Filtering from %s pages" % (len(all_pages))
        if self.options.page:
            pages = [self.options.page]
        else:
#RS continue

##            pages = list(wikiutil.getPageList(config.text_dir))
#RS filter pagelist by pattern
            query = search.QueryParser(regex=1, titlesearch=1,
                                       case=1).parse_query(page_pattern)
            results = search.searchPages(self.request, query)
            results.sortByPagename()
#            pages= results.pageList(self.request, self.formatter)
            pages= [(hit.page_name) for hit in results.hits]
            _util.log('Found %s of about %s pages matching pattern "%s"...' % (len(results.hits),results.pages,page_pattern))


#RS stop
##            all_pages = list(wikiutil.getPageList(config.text_dir))
##            if page_pattern:
##                try:
##                    needle_re = re.compile(page_pattern, re.IGNORECASE)
##                except re.error, e:
##                    print ('%s' %
##                    _("ERROR in pattern regex '%s'") % (inc_pattern,), e)
##                pages = filter(needle_re.search, all_pages)
##            else:
##                pages=all_pages
#RS end
#        pages.sort()
##        print "Dumping %s pages" % (len(pages))
        
        errfile_name='%s_error.txt' % self.doctype
        errfile = os.path.join(self.outputdir, errfile_name)
        self.errlog = open(errfile, 'w')
        errcnt = 0
        self.request.export_pages=pages
        for pagename in pages:
#RS refactored code
#RS extension handling
#            file = wikiutil.quoteFilename(pagename) + self.ext
            file = wikiutil.quoteWikinameURL(pagename) # we have the same name in URL and FS
#            _util.log('Writing "%s"...' % file)
#RS end
            output=self.dumpPage(pagename,file)
            self.postprocessPage(pagename,file,output)

#RS disabled, HTML only and nonsense for our purpose
#RS end
        self.errlog.close()
        if errcnt:
            print >>sys.stderr, "*** %d error(s) occurred, see '%s'!" % (errcnt, errfile)

    def init_request(self):
        """ create request """
        from MoinMoin.request_ors import RequestExportCLI
        if self.options.wiki_url:
            self.request = RequestExportCLI(self.options.wiki_url, self.options.page)
        else:
            self.request = RequestExportCLI(pagename=self.options.page)
        self.request.form = self.request.args = self.request.setup_args()

##def run():
####    print "this is run from ROBERTS FARM"
##
##    MoinExporter().run()
##
##if __name__ == "__main__":
##    run()
##
##"""
##
##
##
##
##
##"""
